searchNodes=[{"doc":"TripSwitch is an Elixir implementation of a circuit breaker. The circuit breaker is a popular pattern in softwares that depends on various components/services to keep running. This library allows you wrap calls (signals) to remote calls in a trip switch. The switch monitors each invocation and records failures, the trip switch gets broken once the recorded failures reaches or surpasses the specified threshold. Signal A signal is simple a function that gets invoked. The state of the trip switch gets updated based on the result of this signal, see TripSwitch.Breaker.signal/0 . Examples iex&gt; TripSwitch . send ( id , fn -&gt; { :ok , :good } end ) { :ok , :good } iex&gt; TripSwitch . send ( id , fn -&gt; { :break , :good } end ) # record this signal as a failure { :ok , :good } Note that a switch only stops invoking a signal after the failure threshold have been reached. In this case, the switch keeps is considered broken until it's repaired. Repairs It's obvious you are asking yourself what you should do when a trip switch gets broken. This library have a concept called Repair. What this means is that a switch is sent for repair automatically once it gets broken. Not all trip switches are considered fixable, you need to specify a repair_time (in milliseconds) when the trip switch is created else you need to manually fix (reset) the trip switch whenever it gets broken. Thresholds For a trip switch to function correctly, a threshold is needed to be specified. This allows us to define the capacity/expectation of the trip switch. The threshold is specified as a float value, usually between 0 and 1 (ex. 0.2566 or 0.13 and so on). Simply, a threshold is the percentage of bad signals a trip switch receives before it gets broken and sent for repair. Usage To create a trip switch you need to add it to your supervision tree: children = [ { TripSwitch , name : :switch , threshold : 0.5 } ] Supervisor . start_link ( children , opts ) It supports following options: :name - this will be used as the id for the switch :threshold - the maximum number of thresholds before the switch trips :repair_time - time taken for the switch to get fixed after it gets broken Telemetry Being a big fan of observability, this library exposes information about it internals using the community approved library ( telemetry ). You can use the telemetry_poller library to poll these metrics. Below are events that this library publishes. Below are events published by trip_switch : [:trip_switch, :signal, :start] - dispatched before a given signal is handled Measurement: %{system_time: system_time} Metadata: %{id: atom(), tag: String.t()} [:trip_switch, :signal, :stop] - dispatched after a given signal have been handled Measurement: %{duration: native_time} Metadata: %{id: atom(), tag: String.t()} [:trip_switch, :repair, :start] - dispatched when auto-repair is scheduled Measurement: %{system_time: system_time} Metadata: %{id: atom(), tag: String.t()} [:trip_switch, :repair, :stop] - dispatched after an auto-repair have been completed Measurement: %{duration: native_time} Metadata: %{id: atom(), tag: String.t()}","ref":"TripSwitch.html","title":"TripSwitch","type":"module"},{"doc":"Checks if the switch is broken. A switch is considered broken if the underlying breaker is half_open or open . Examples iex&gt; TripSwitch . broken? ( :switch ) true","ref":"TripSwitch.html#broken?/1","title":"TripSwitch.broken?/1","type":"function"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"TripSwitch.html#child_spec/1","title":"TripSwitch.child_spec/1","type":"function"},{"doc":"Reset the given switch. Calling this function on a broken switch returns it to a working state. Examples iex&gt; TripSwitch . reset ( :switch ) :ok","ref":"TripSwitch.html#reset/1","title":"TripSwitch.reset/1","type":"function"},{"doc":"Send a signal to the underlying breaker. The signal is a function that performs some actions then return a result that alters the state of the underlying breaker. Examples iex&gt; TripSwitch . send ( :switch , fn -&gt; { :ok , %{ name : &quot;a&quot; } } ) end ) { :ok , %{ name : &quot;a&quot; } } iex&gt; TripSwitch . send ( :switch , fn -&gt; { :break , { :error , :not_found } } end ) { :error , :not_found }","ref":"TripSwitch.html#send/2","title":"TripSwitch.send/2","type":"function"},{"doc":"Documentation for TripSwitch.Breaker .","ref":"TripSwitch.Breaker.html","title":"TripSwitch.Breaker","type":"module"},{"doc":"","ref":"TripSwitch.Breaker.html#broken?/1","title":"TripSwitch.Breaker.broken?/1","type":"function"},{"doc":"","ref":"TripSwitch.Breaker.html#handle/2","title":"TripSwitch.Breaker.handle/2","type":"function"},{"doc":"","ref":"TripSwitch.Breaker.html#new/1","title":"TripSwitch.Breaker.new/1","type":"function"},{"doc":"","ref":"TripSwitch.Breaker.html#repair/1","title":"TripSwitch.Breaker.repair/1","type":"function"},{"doc":"","ref":"TripSwitch.Breaker.html#repairable?/1","title":"TripSwitch.Breaker.repairable?/1","type":"function"},{"doc":"","ref":"TripSwitch.Breaker.html#reset/1","title":"TripSwitch.Breaker.reset/1","type":"function"},{"doc":"","ref":"TripSwitch.Breaker.html#t:signal/0","title":"TripSwitch.Breaker.signal/0","type":"type"},{"doc":"","ref":"TripSwitch.Breaker.html#t:state/0","title":"TripSwitch.Breaker.state/0","type":"type"},{"doc":"","ref":"TripSwitch.Breaker.html#t:t/0","title":"TripSwitch.Breaker.t/0","type":"type"},{"doc":"TODO: Add description","ref":"readme.html","title":"TripSwitch","type":"extras"},{"doc":"If available in Hex , the package can be installed by adding trip_switch to your list of dependencies in mix.exs : def deps do [ { :trip_switch , &quot;~&gt; 0.1.3&quot; } ] end Documentation can be generated with ExDoc and published on HexDocs . Once published, the docs can be found at https://hexdocs.pm/trip_switch .","ref":"readme.html#installation","title":"TripSwitch - Installation","type":"extras"},{"doc":"An alias/command is included to make a new release of the package. # include --initial for first release and without for subsequent releases mix ops.release","ref":"readme.html#release","title":"TripSwitch - Release","type":"extras"},{"doc":"MIT License Copyright (c) 2023 Atanda Rasheed Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"}]